@@ -170,7 +170,7 @@ class BParseOptionsTestCase(unittest.TestCase):
 
     def test_setting_verbosity_to_not_allow_value_raises_value_error(self):
         po = ParseOptions()
-        self.assertRaises(ValueError, setattr, po, "verbosity", 121)
+        self.assertRaises(ValueError, setattr, po, "verbosity", -1)
 
     def test_setting_verbosity_to_non_integer_raises_type_error(self):
         po = ParseOptions()
@@ -352,7 +352,7 @@ class DBasicParsingTestCase(unittest.TestCase):
     def test_that_parse_returns_empty_iterator_on_no_linkage_sat(self):
         """Parsing a bad sentence with no null-links shouldn't give any linkage (sat)"""
         self.po = ParseOptions(use_sat=True)
-        if self.po.use_sat != True:
+        if not self.po.use_sat:
             raise unittest.SkipTest("Library not configured with SAT parser")
         result = self.parse_sent("This this doesn't parse", self.po)
         linkage_exists = False
@@ -583,7 +583,6 @@ class EErrorFacilityTestCase(unittest.TestCase):
 
     def test_50_set_orig_error_handler(self):
         # Set the error handler back to the default handler.
-        # The error message is now visible (but we cannot test that).
         self.__class__.handler["previous"] = LG_Error.set_handler(self.__class__.handler["default"])
         self.assertIsNone(self.__class__.handler["previous"])
         for _ in range(0, 1 + self.testleaks):
@@ -601,7 +600,7 @@ class FSATsolverTestCase(unittest.TestCase):
     def setUp(self):
         self.d, self.po = Dictionary(lang='en'), ParseOptions()
         self.po = ParseOptions(use_sat=True)
-        if self.po.use_sat != True:
+        if not self.po.use_sat:
             raise unittest.SkipTest("Library not configured with SAT parser")
 
     def test_SAT_getting_links(self):
@@ -672,7 +671,7 @@ class HEnglishLinkageTestCase(unittest.TestCase):
     # -- the one that is in the dict is not the grammatically appropriate word.
     #
     # Let's is NOT split into two! It's in the dict as one word, lower-case only.
-    def test_f_captilization(self):
+    def test_f_capitalization(self):
         self.assertEqual(list(self.parse_sent('Let\'s eat.')[0].words()),
              ['LEFT-WALL', 'let\'s', 'eat.v', '.', 'RIGHT-WALL'])
 
@@ -969,6 +968,149 @@ class JBDictCostReadingTestCase(unittest.TestCase):
         linkage = Sentence('Is the bed white?', self.d, self.po).parse().next()
         self.assertEqual(list(linkage.words())[4], 'white.a')
 
+
+def sm(s):
+    """
+    Naive '.' replacement by SUBSCRIPT_MARK (enough for the tests here).
+    """
+    SUBSCRIPT_MARK = '\3'
+    return s.replace('.', SUBSCRIPT_MARK)
+
+
+class XLookupListTestCase(unittest.TestCase):
+    """
+    Lookup-list related tests.
+    Note: In case of test errors, the subscript mark in words may be mis-printed.
+    """
+    @classmethod
+    def setUpClass(cls):
+        cls.d_en, cls.d_sql, cls.po = Dictionary(lang='en'), Dictionary(lang='demo-sql'), ParseOptions()
+
+    @classmethod
+    def tearDownClass(cls):
+        del cls.d_en, cls.d_sql, cls.po
+
+    def test_file_lookup_list_none(self):
+        self.assertIsNone(clg.dictionary_lookup_list(self.d_en._obj, 'NoSuchWord'))
+
+    def test_file_lookup_wild_none(self):
+        self.assertIsNone(clg.dictionary_lookup_wild(self.d_en._obj, 'NoSuch*'))
+
+    def test_sql_lookup_list_none(self):
+        self.assertIsNone(clg.dictionary_lookup_list(self.d_sql._obj, 'NoSuchWord'))
+
+    def test_sql_lookup_wild_none(self):
+        self.assertIsNone(clg.dictionary_lookup_wild(self.d_sql._obj, 'NoSuch*'))
+
+    def test_file_lookup_list_subscr(self):
+        dictnode = clg.dictionary_lookup_list(self.d_en._obj, sm('test.n'))
+        self.assertEqual(dictnode[0].string, sm('test.n'))
+
+    def test_file_lookup_list_no_subscr(self):
+        dictnode = clg.dictionary_lookup_list(self.d_en._obj, 'test')
+        self.assertEqual(sorted([dictnode[i].string for i in range(len(dictnode))]), [sm('test.n'), sm('test.v')])
+        for i in range(len(dictnode)):
+            self.assertIn("(", str(dictnode[i].exp), "Missing expression")
+            if dictnode[i].string == sm('test.n'):
+                self.assertEqual(dictnode[i].file, 'en/words/words.n.1-const')
+            elif dictnode[i].string == sm('test.v'):
+                self.assertIsNone(dictnode[i].file)
+
+    @unittest.skip("FIXME: Cannot lookup with SUBSCRIPT_MARK")
+    def test_sql_lookup_list_subscr(self):
+        dictnode = clg.dictionary_lookup_list(self.d_en._obj, sm('test.n'))
+        self.assertEqual(dictnode[0].string, sm('test.n'))
+
+    @unittest.skip("FIXME: It returns a dot subscript instead of SUBSCRIPT_MARK")
+    def test_sql_lookup_list_no_subscr(self):
+        dictnode = clg.dictionary_lookup_list(self.d_sql._obj, 'test')
+        self.assertEqual(sorted([dictnode[i].string for i in range(len(dictnode))]), [sm('test.n')])
+
+    def test_file_lookup_wild_any_subscript(self):
+        dictnode = clg.dictionary_lookup_wild(self.d_en._obj, 'test*')
+        self.assertTrue(len(dictnode) > 40, 'Missing words (only {} found)'.format(len(dictnode)))
+        for dn in dictnode:
+            self.assertIsNotNone(re.search('test.*', dn.string), 'Bad word {}'.format(dn.string))
+
+    def test_file_lookup_wild_n_subscript(self):
+        dictnode = clg.dictionary_lookup_wild(self.d_en._obj, 'test*.n')
+        self.assertTrue(len(dictnode) > 10, 'Missing words (only {} found)'.format(len(dictnode)))
+        for dn in dictnode:
+            self.assertIsNotNone(re.search('test.*' + sm('.') + 'n', dn.string), 'Bad word {}'.format(dn.string))
+
+    def test_sql_lookup_wild_any_subscript(self):
+        dictnode = clg.dictionary_lookup_wild(self.d_en._obj, 't*')
+        self.assertTrue(len(dictnode) > 40, 'Missing words (only {} found)'.format(len(dictnode)))
+        for dn in dictnode:
+            self.assertIsNotNone(re.search('t.*', dn.string), 'Bad word {}'.format(dn.string))
+
+
+# Noun expressions currently contain only a dialect component "headline".
+# It controls the cost of a null expression in 2 places.
+class XExp_resolving_test(unittest.TestCase):
+    @classmethod
+    def setUpClass(cls):
+        cls.d = Dictionary(lang='en')
+
+    @classmethod
+    def tearDownClass(cls):
+        del cls.d
+
+    def test_exp_copying(self):
+        dictnode = clg.dictionary_lookup_list(self.d._obj, sm('test.n'))
+        exp_old = dictnode[0].exp
+        exp_new = clg.lg_exp_resolve(self.d._obj, exp_old)
+        self.assertEqual(str(exp_old), str(exp_new))
+
+    # When a zero cost is assigned to "headline", the expression should remain the same.
+    def test_no_op_resolving(self):
+        dictnode = clg.dictionary_lookup_list(self.d._obj, sm('test.n'))
+        exp_old = dictnode[0].exp
+        exp_new = clg.lg_exp_resolve(self.d._obj, exp_old, ParseOptions(dialect="headline:0")._obj)
+        self.assertEqual(str(exp_old), str(exp_new))
+
+    def test_resolving(self):
+        """
+        Test expression resolving using the default headline:4 setting from
+        data/en/4.0.dialect.
+        """
+        dictnode = clg.dictionary_lookup_list(self.d._obj, sm('test.n'))
+        exp_old = dictnode[0].exp
+        exp_new = clg.lg_exp_resolve(self.d._obj, exp_old, ParseOptions()._obj)  # headline:4
+
+        # Find the 2 locations with a difference when comparing
+        # exp_old to exp_new and validate them.
+        # Locations 2 and 4 are in exp_old. Locations 6 and 7 are in exp_new.
+        # () is the null expression that is controlled by "headline".
+        str_comb = str(exp_old) + '%' + str(exp_new)
+        diff = re.search(r'^([^%]*)(\(Ds\*\*x- or \([^%]*?\)\)\)\))([^%]*)(\(Ds\*\*c- or \([^%]*?\)\)\)\))([^%]*)%\1(.*)\3(.*)\5$', str_comb)
+        #                      1                2                     3                4                     5        6     7
+        self.assertEqual(diff.group(2), '(Ds**x- or (())))')
+        self.assertEqual(diff.group(4), '(Ds**c- or (())))')
+        self.assertEqual(diff.group(6), '(Ds**x- or ([[[[()]]]])))')
+        self.assertEqual(diff.group(7), '(Ds**c- or ([[[[()]]]])))')
+
+
+# Currently, the dictionary creating function sets the generation mode if
+# the "test" parse-option has "generate" in its value list. So it must be
+# set so before the call to Dictionary(). When the second argument of
+# Sentence is evaluated, it initializes the test parse-option to a null
+# string.
+class YGenerationTestCase(unittest.TestCase):
+    """
+    Generation mode tests.
+    """
+    def test_getting_linkages_file_dict(self):
+        ParseOptions(test='generate')
+        linkages = Sentence((clg.WILDCARD_WORD + ' ') * 5, Dictionary(lang='lt'), ParseOptions()).parse()
+        self.assertTrue(len(linkages) > 0, "No linkages")
+
+    def test_getting_linkages_sql_dict(self):
+        ParseOptions(test='generate')
+        linkages = Sentence((clg.WILDCARD_WORD + ' ') * 4, Dictionary(lang='demo-sql'), ParseOptions()).parse()
+        self.assertTrue(len(linkages) > 0, "No linkages")
+
+
 class ZENConstituentsCase(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
